namespace Eco.Mods.TechTree
{
    using Eco.Gameplay.Components;
    using Eco.Gameplay.Components.Auth;
    using Eco.Gameplay.DynamicValues;
    using Eco.Gameplay.Housing;
    using Eco.Gameplay.Items;
    using Eco.Gameplay.Objects;
    using Eco.Gameplay.Players;
    using Eco.Gameplay.Property;
    using Eco.Gameplay.Skills;
    using Eco.Gameplay.Systems.TextLinks;
    using Eco.Gameplay.Systems.Tooltip;
    using Eco.Shared.Localization;
    using Eco.Shared.Serialization;
    using System;

    [Serialized]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(MinimapComponent))]
    [RequireComponent(typeof(LinkComponent))]
    [RequireComponent(typeof(CraftingComponent))]
    [RequireComponent(typeof(HousingComponent))]
    [RequireComponent(typeof(SolidGroundComponent))]
    [RequireComponent(typeof(RoomRequirementsComponent))]
    [RequireComponent(typeof(PublicStorageComponent))]
    [RequireRoomContainment]
    [RequireRoomVolume(25)]
    public partial class WineBarrelObject : WorldObject
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Wine Barrel"); } }


        protected override void Initialize()
        {
            this.GetComponent<MinimapComponent>().Initialize("Cooking");
            this.GetComponent<HousingComponent>().Set(WineBarrelItem.HousingVal);
            var storage = this.GetComponent<PublicStorageComponent>();
            storage.Initialize(2);
            storage.Storage.AddInvRestriction(new CustomRestriction(typeof(FermentedBlueberryItem)));
        }

        public override void Destroy()
        {
            base.Destroy();
        }

    }

    [Serialized]
    [MaxStackSize(1)]
    [Weight(18000)]
    public partial class WineBarrelItem :
        WorldObjectItem<WineBarrelObject>
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Wine Barrel"); } }
        public override LocString DisplayDescription { get { return Localizer.DoStr("Used for making various drinks"); } }

        static WineBarrelItem()
        {

        }

        [TooltipChildren] public HousingValue HousingTooltip { get { return HousingVal; } }
        [TooltipChildren]
        public static HousingValue HousingVal
        {
            get
            {
                return new HousingValue()
                {
                    Category = "Kitchen",
                    Val = 3,
                    TypeForRoomLimit = "Cooking",
                    DiminishingReturnPercent = 0.3f
                };
            }
        }
    }

    [RequiresSkill(typeof(HewingSkill), 4)]
    public partial class WineBarrelRecipe : Recipe
    {
        public WineBarrelRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<WineBarrelItem>(),
            };

            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<BoardItem>(typeof(HewingSkill), 50, HewingSkill.MultiplicativeStrategy),
                new CraftingElement<IronIngotItem>(typeof(SmeltingSkill), 5, SmeltingSkill.MultiplicativeStrategy),
            };
            
            this.CraftMinutes = Recipe.CreateCraftTimeValue(typeof(WineBarrelRecipe), Item.Get<WineBarrelItem>().UILink(), 30f, typeof(HewingSkill), typeof(HewingFocusedSpeedTalent));
            this.Initialize(Localizer.DoStr("Wine Barrel"), typeof(WineBarrelRecipe));
            CraftingComponent.AddRecipe(typeof(CarpentryTableObject), this);
        }
    }

    [Serialized]
    [Weight(1200)]
    public partial class BlueberryWineItem :
        FoodItem
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Blueberry Wine"); } }
        public override LocString DisplayDescription { get { return Localizer.DoStr("A surprisingly sweet taste"); } }

        private static Nutrients nutrition = new Nutrients() { Carbs = 12, Fat = 0, Protein = 6, Vitamins = 20 };
        public override float Calories { get { return 750; } }
        public override Nutrients Nutrition { get { return nutrition; } }
    }

    [RequiresSkill(typeof(AdvancedCookingSkill), 2)]
    public partial class BlueberryWineRecipe : Recipe
    {
        public BlueberryWineRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<BlueberryWineItem>(),

            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<EmptyBottleItem>(typeof(AdvancedCookingSkill), 1, AdvancedCookingSkill.MultiplicativeStrategy),
                new CraftingElement<FermentedBlueberryItem>(typeof(AdvancedCookingSkill), 3, AdvancedCookingSkill.MultiplicativeStrategy),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(BlueberryWineRecipe), Item.Get<BlueberryWineItem>().UILink(), 10, typeof(AdvancedCookingSkill), typeof(AdvancedCookingFocusedSpeedTalent));
            this.Initialize(Localizer.DoStr("Blueberry Wine"), typeof(BlueberryWineRecipe));
            CraftingComponent.AddRecipe(typeof(WineBarrelObject), this);
        }
    }

    [RequiresSkill(typeof(GlassworkingSkill), 1)]
    public partial class EmptyBottleRecipe : Recipe
    {
        public EmptyBottleRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<EmptyBottleItem>(1),
            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<GlassItem>(typeof(GlassworkingSkill), 5, GlassworkingSkill.MultiplicativeStrategy),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(EmptyBottleRecipe), Item.Get<EmptyBottleItem>().UILink(), 10, typeof(GlassworkingSkill), typeof(GlassworkingFocusedSpeedTalent));
            this.Initialize(Localizer.DoStr("Empty wine bottle"), typeof(EmptyBottleRecipe));

            CraftingComponent.AddRecipe(typeof(KilnObject), this);
        }
    }


    [Serialized]
    [Weight(200)]
    public partial class EmptyBottleItem :
    Item
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Empty wine bottle"); } }
        public override LocString DisplayDescription { get { return Localizer.DoStr("An empty bottle is used to create blueberry wine craft an wine barrel."); } }

    }

    [Serialized]
    [Weight(200)]
    public partial class FermentedBlueberryItem : FoodItem
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Fermented Blueberry"); } }
        public override LocString DisplayDescription { get { return Localizer.DoStr("Fermented juice in made at the Wine Barrel and can then be used for the production of other varieties"); } }

        private static Nutrients nutrition = new Nutrients() { Carbs = 5, Fat = 0, Protein = 0, Vitamins = 20 };
        public override float Calories { get { return 120; } }
        public override Nutrients Nutrition { get { return nutrition; } }
    }

    [RequiresSkill(typeof(AdvancedCookingSkill), 1)]
    public partial class FermentedBlueberryRecipe : Recipe
    {
        public FermentedBlueberryRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<FermentedBlueberryItem>(),

            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<HuckleberriesItem>(typeof(AdvancedCookingSkill), 100, AdvancedCookingSkill.MultiplicativeStrategy),
                new CraftingElement<SugarItem>(typeof(AdvancedCookingSkill), 10, AdvancedCookingSkill.MultiplicativeStrategy),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(FermentedBlueberryRecipe), Item.Get<FermentedBlueberryItem>().UILink(), 60, typeof(AdvancedCookingSkill), typeof(AdvancedCookingFocusedSpeedTalent));
            this.Initialize(Localizer.DoStr("Fermented Blueberry"), typeof(FermentedBlueberryRecipe));
            CraftingComponent.AddRecipe(typeof(WineBarrelObject), this);
        }
    }

    //======================================================================

    public class CustomRestriction : InventoryRestriction
    {
        private Type acceptedItem;

        private string message = "";
        public override string Message
        {
            get
            {
                return this.message;
            }
        }

        public CustomRestriction(Type acceptedItem)
        {
            this.acceptedItem = acceptedItem;
        }

        public override int MaxAccepted(Item item, int currentQuantity)
        {
            if (item.Type != acceptedItem)
            {
                message = "You can only put the following item: " + Item.Get(acceptedItem).UILink();
                return 0;
            }
            return item.MaxStackSize;
        }
    }
}